#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "avg_a.h"
#include "avg_b.h"
#include "avg_c.h"
#include "util.h"


// **************************************************************************
//
// **************************************************************************
#define SIZE	(1 * 1024)
#define REPEAT	800000
int buffer[SIZE];


// **************************************************************************
//
// **************************************************************************
static void test_avg(void)
{
	double avg1, avg2, total = 0.0;

	random_ints(buffer, SIZE);

	uint64_t t0 = now();

	for (int r = 0; r < REPEAT; r++) {
		averages_A(buffer, SIZE, &avg1, &avg2);
		total += avg1 + avg2;
	}

	uint64_t t1 = now();

	for (int r = 0; r < REPEAT; r++) {
		averages_B(buffer, SIZE, &avg1, &avg2);
		total += avg1 + avg2;
	}

	uint64_t t2 = now();

	for (int r = 0; r < REPEAT; r++) {
		averages_C(buffer, SIZE, &avg1, &avg2);
		total += avg1 + avg2;
	}

	uint64_t t3 = now();

	printf("%.1f\n\n", total / (3.0 * REPEAT));

	uint64_t d1 = t1 - t0;
	uint64_t d2 = t2 - t1;
	uint64_t d3 = t3 - t2;

	double r2 = (double)d2 / d1;
	double r3 = (double)d3 / d1;

	printf("1: %10ld  %.3fx\n", d1, 1.0);
	printf("2: %10ld  %.3fx\n", d2, r2);
	printf("3: %10ld  %.3fx\n", d3, r3);
}

// **************************************************************************
//
// **************************************************************************
void tr_A(char *s)
{
	while (*s != 0) {
		if ((*s >= 0) && (*s <= 31))
			*s = '#';

		s++;
	}
}

void tr_B(char *s)
{
	while (*s != 0) {
		int v = *((uint8_t *)s);
		int mask = (v - 32) >> 31;
		*s = (v & (~mask)) | ('#' & mask);

		s++;
	}
}


#define TR_SIZE		4096
#define TR_REPEAT	100000
char tr_random[TR_SIZE];
char tr_buffer[TR_SIZE];

static void test_tr(void)
{
	random_chars(tr_random, TR_SIZE);

	uint64_t t0 = now();

	for (int r = 0; r < TR_REPEAT; r++) {
		memcpy(tr_buffer, tr_random, TR_SIZE);
		tr_B(tr_buffer);
	}

	uint64_t t1 = now();

	for (int r = 0; r < TR_REPEAT; r++) {
		memcpy(tr_buffer, tr_random, TR_SIZE);
		tr_A(tr_buffer);
	}

	uint64_t t2 = now();

	uint64_t t3 = now();

	uint64_t d1 = t1 - t0;
	uint64_t d2 = t2 - t1;
	uint64_t d3 = t3 - t2;

	double r2 = (double)d2 / d1;
	double r3 = (double)d3 / d1;

	printf("1: %10ld  %.3fx\n", d1, 1.0);
	printf("2: %10ld  %.3fx\n", d2, r2);
	printf("3: %10ld  %.3fx\n", d3, r3);
}

// **************************************************************************
//
// **************************************************************************
int main(int argc, char **argv)
{
	if (argc < 2) {
		fprintf(stderr, "Usage: benchmark tr | avg\n");
		return 1;
	}

	if (strcmp(argv[1], "tr") == 0) {
		test_tr();
	} else if (strcmp(argv[1], "avg") == 0) {
		test_avg();
	} else {
		fprintf(stderr, "Usage: benchmark tr | avg\n");
		return 1;
	}

	return 0;
}

// **************************************************************************
//
// **************************************************************************

