#include <stdio.h>
#include <stdlib.h>

#define REPEAT	1
#define SIZE	8192

double *matrix_alloc(void)
{
	double *a = malloc(SIZE * SIZE * sizeof(double));

	if (a == NULL)
		perror("matrix_alloc()");

	return a;
}

void matrix_random(double *a)
{
	static unsigned long r = 0x92a834c7df2e983d;

	for (int i = 0; i < SIZE; i++) {
		for (int j = 0; j < SIZE; j++) {
			r = (r * 25214903917 + 11) & 0xffffffffffff;
			a[i * SIZE + j] = (double)(r >> 16) / 0xffffffff;
		}
	}
}

double matrix_sum(const double *a)
{
	double sum = 0.0;

	for (int i = 0; i < SIZE; i++) {
		for (int j = 0; j < SIZE; j++) {
			sum = sum + a[i * SIZE + j];
		}
	}

	return sum;
}

void matrix_multiply(double *x, const double *a, const double *b)
{
	for (int i = 0; i < SIZE; i++) {
		for (int j = 0; j < SIZE; j++) {
			x[i * SIZE + j] = 0.0;
		}
	}
}

int main(void)
{
	double *a = matrix_alloc();
	double *b = matrix_alloc();
	double *x = matrix_alloc();

	matrix_random(a);
	matrix_random(b);

	for (int r = 0; r < REPEAT; r++) {
		matrix_multiply(x, a, b);
	}

	printf("%g\n", matrix_sum(x));

	free(a);
	free(b);
	free(x);

	return 0;
}
