#include <stdio.h>
#include <stdlib.h>
#include <openblas/cblas.h>

#define REPEAT	1
#define SIZE	32768

float *matrix_alloc(void)
{
	float *a = malloc(SIZE * SIZE * sizeof(float));

	if (a == NULL)
		perror("matrix_alloc()");

	return a;
}

void matrix_random(float *a)
{
	static unsigned long r = 0x92a834c7df2e983d;

	for (int i = 0; i < SIZE; i++) {
		for (int j = 0; j < SIZE; j++) {
			r = (r * 25214903917 + 11) & 0xffffffffffff;
			a[i * SIZE + j] = (float)(r >> 16) / 0xffffffff;
		}
	}
}

float matrix_sum(const float *a)
{
	float sum = 0.0;

	for (int i = 0; i < SIZE; i++) {
		for (int j = 0; j < SIZE; j++) {
			sum = sum + a[i * SIZE + j];
		}
	}

	return sum;
}

void matrix_multiply4(float *restrict x,
	const float *restrict a, const float * restrict b)
{
	cblas_sgemm(CblasRowMajor, CblasNoTrans, CblasTrans,
		SIZE, SIZE, SIZE, 1.0, a, SIZE, b, SIZE,
		0.0, x, SIZE);
}


int main(void)
{
	float *a = matrix_alloc();
	float *b = matrix_alloc();
	float *x = matrix_alloc();

	matrix_random(a);
	matrix_random(b);

	for (int r = 0; r < REPEAT; r++) {
		matrix_multiply4(x, a, b);
	}

	printf("%g\n", matrix_sum(x));

	free(a);
	free(b);
	free(x);

	return 0;
}

